/* GTK - The GIMP Toolkit
 * Copyright (C) 2000 Red Hat Software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 * Inscript Keymap for Bengali
 * 
 * Author(s)  G Karunakar  <karunakarg@hotmail.com>
 *  
 */

#include <string.h>

#include <gdk/gdkkeysyms.h>

#include "gtk/gtkintl.h"
#include "gtk/gtkimcontextsimple.h"
#include "gtk/gtkimmodule.h"

#define INSCRIPT_BNG_IM_ID  "inscript_bng_im"

GType type_inscript_bng_im = 0;

static void inscript_bng_im_class_init (GtkIMContextSimpleClass *class);
static void inscript_bng_im_init (GtkIMContextSimple *im_context);

static void
inscript_bng_im_register_type (GTypeModule *module)
{
  static const GTypeInfo object_info =
  {
    sizeof (GtkIMContextSimpleClass),
    (GBaseInitFunc) NULL,
    (GBaseFinalizeFunc) NULL,
    (GClassInitFunc) inscript_bng_im_class_init,
    NULL,           /* class_finalize */
    NULL,           /* class_data */
    sizeof (GtkIMContextSimple),
    0,
    (GtkObjectInitFunc) inscript_bng_im_init,
  };

  type_inscript_bng_im = 
    g_type_module_register_type (module,
				 GTK_TYPE_IM_CONTEXT_SIMPLE,
				 "GtkIMContextBengali",
				 &object_info, 0);
}


/* Based on the Inscript keyboard layout for devanagari
 *  Commented Keysyms are unassigned, leaving them so that can assign later on
 *  Some conjuncts are also assigned to keys, not yet figured how to get them
 */

static guint16 inscript_bng_compose_seqs[] = {

/*  GDK_exclam,		0,	0,	0,	0,	0x090D,  LETTER CANDRA E */
  GDK_quotedbl,		0,	0,	0,	0,	0x09A0, /* LETTER TTHA     */
/*  GDK_numbersign,	0,	0,	0,	0,	0x09,  VIRAMA + LETTER RA  */
/*  GDK_dollar,		0,	0,	0,	0,	0x09,  LETTER RA + VIRAMA  */
/*  GDK_percent,	0,	0,	0,	0,	0x09,  LETTER JA + VIRAMA + LETTER NYA */
/*  GDK_ampersand,	0,	0,	0,	0,	0x09,  LETTER KA + VIRAMA + LETTER SSA */
  GDK_apostrophe,	0,	0,	0,	0,	0x099F, /* LETTER TTA */
/* GDK_parenleft,	0,	0,	0,	0,	0x09,   */
/* GDK_parenright,	0,	0,	0,	0,	0x09,   */
/* GDK_asterisk,	0,	0,	0,	0,	0x09,  LETTER SHA + VIRAMA + LETTER RA */
  GDK_plus,		0,	0,	0,	0,	0x098B, /* LETTER VOC R */
/* GDK_comma,		0,	0,	0,	0,	0x09,    */
/* GDK_minus,		0,	0,	0,	0,	0x09,    */
/* GDK_period,		0,	0,	0,	0,	0x09,    */
  GDK_slash,		0,	0,	0,	0,	0x9AF, /* LETTER YA */

  GDK_0,	0,	0,	0,	0,	0x09E6,  /* DIGIT 0 */
  GDK_1,	0,	0,	0,	0,	0x09E7,  /* DIGIT 1 */ 
  GDK_2,	0,	0,	0,	0,	0x09E8,  /* DIGIT 2 */ 
  GDK_3,	0,	0,	0,	0,	0x09E9,  /* DIGIT 3 */ 
  GDK_4,	0,	0,	0,	0,	0x09EA,  /* DIGIT 4 */ 
  GDK_5,	0,	0,	0,	0,	0x09EB,  /* DIGIT 5 */ 
  GDK_6,	0,	0,	0,	0,	0x09EC,  /* DIGIT 6 */ 
  GDK_7,	0,	0,	0,	0,	0x09ED,  /* DIGIT 7 */ 
  GDK_8,	0,	0,	0,	0,	0x09EE,  /* DIGIT 8 */ 
  GDK_9,	0,	0,	0,	0,	0x09EF,  /* DIGIT 9 */ 
  
  GDK_colon,		0,	0,	0,	0,	0x099B, /* LETTER CHA  */
  GDK_semicolon,	0,	0,	0,	0,	0x099A, /* LETTER CA   */
  GDK_less,		0,	0,	0,	0,	0x09B7, /* LETTER SSA  */
  GDK_equal,		0,	0,	0,	0,	0x09C3, /* VS VOC R    */
  GDK_greater,		0,	0,	0,	0,	0x0964, /* DEV DANDA  */
  GDK_question,		0,	0,	0,	0,	0x09DF, /* LETTER YYA  */
/* GDK_at,		0,	0,	0,	0,	0x09,    */

  GDK_A,	0,	0,	0,	0,	0x0993, /* LETTER O */
/* GDK_B,	0,	0,	0,	0,	0x09,    */
  GDK_C,	0,	0,	0,	0,	0x09A3, /* LETTER NNA */
  GDK_D,	0,	0,	0,	0,	0x0985, /* LETTER A */
  GDK_E,	0,	0,	0,	0,	0x0986, /* LETTER AA */
  GDK_F,	0,	0,	0,	0,	0x0987, /* LETTER I */
  GDK_G,	0,	0,	0,	0,	0x0989, /* LETTER U */
  GDK_H,	0,	0,	0,	0,	0x09AB, /* LETTER PHA */
  GDK_I,	0,	0,	0,	0,	0x0998, /* LETTER GHA */
  GDK_J,	0,	0,	0,	0,	0x09DC, /* LETTER RRA */
  GDK_K,	0,	0,	0,	0,	0x0996, /* LETTER KHA */
  GDK_L,	0,	0,	0,	0,	0x09A5, /* LETTER THA */
  GDK_M,	0,	0,	0,	0,	0x09B6, /* LETTER SHA */
/* GDK_N,	0,	0,	0,	0,	0x09,    */
  GDK_O,	0,	0,	0,	0,	0x09A7, /* LETTER DHA */
  GDK_P,	0,	0,	0,	0,	0x099D, /* LETTER JHA */
  GDK_Q,	0,	0,	0,	0,	0x0994, /* LETTER AU */
  GDK_R,	0,	0,	0,	0,	0x0988, /* LETTER II */
  GDK_S,	0,	0,	0,	0,	0x098F, /* LETTER E */
  GDK_T,	0,	0,	0,	0,	0x098A, /* LETTER UU */
  GDK_U,	0,	0,	0,	0,	0x0999, /* LETTER NGA */
/* GDK_V,	0,	0,	0,	0,	0x09,    */
  GDK_W,	0,	0,	0,	0,	0x0990, /* LETTER AI */
  GDK_X,	0,	0,	0,	0,	0x0981, /* CANDRABINDU */
  GDK_Y,	0,	0,	0,	0,	0x09AD, /* LETTER BHA */
/* GDK_Z,	0,	0,	0,	0,	0x09,    */

  GDK_bracketleft,	0,	0,	0,	0,	0x9A1, /* LETTER DDA  */
/* GDK_backslash,	0,	0,	0,	0,	0x9,    */
  GDK_bracketright,	0,	0,	0,	0,	0x93C, /* VS NUKTA    */
/* GDK_asciicircum,	0,	0,	0,	0,	0x9		*/
  GDK_underscore,	0,	0,	0,	0,	0x983, /* LETTER VISARGA  */  
/* GDK_grave,		0,	0,	0,	0,	0x9,    */

  GDK_a,	0,	0,	0,	0,	0x09CB, /* VS O */
/* GDK_b,	0,	0,	0,	0,	0x0935,  */
  GDK_c,	0,	0,	0,	0,	0x09AE, /* LETTER MA */
  GDK_d,	0,	0,	0,	0,	0x09CD, /* VIRAMA */
  GDK_e,	0,	0,	0,	0,	0x09BE, /* VS AA */
  GDK_f,	0,	0,	0,	0,	0x09BF, /* VS I */
  GDK_g,	0,	0,	0,	0,	0x09C1, /* VS U */
  GDK_h,	0,	0,	0,	0,	0x09AA, /* LETTER PA */
  GDK_i,	0,	0,	0,	0,	0x0997, /* LETTER GA */
  GDK_j,	0,	0,	0,	0,	0x09B0, /* LETTER RA */
  GDK_k,	0,	0,	0,	0,	0x0995, /* LETTER KA */
  GDK_l,	0,	0,	0,	0,	0x09A4, /* LETTER TA */
  GDK_m,	0,	0,	0,	0,	0x09B8, /* LETTER SA */
  GDK_n,	0,	0,	0,	0,	0x09B2, /* LETTER LA */
  GDK_o,	0,	0,	0,	0,	0x09A6, /* LETTER DA */
  GDK_p,	0,	0,	0,	0,	0x099C, /* LETTER JA */
  GDK_q,	0,	0,	0,	0,	0x0994, /* LETTER AU */
  GDK_r,	0,	0,	0,	0,	0x09C0, /* VS II */
  GDK_s,	0,	0,	0,	0,	0x09C7, /* VS E */
  GDK_t,	0,	0,	0,	0,	0x09C2, /* VS UU */
  GDK_u,	0,	0,	0,	0,	0x09B9, /* LETTER HA */
  GDK_v,	0,	0,	0,	0,	0x09A8, /* LETTER NA */
  GDK_w,	0,	0,	0,	0,	0x09C8, /* VS AI */
  GDK_x,	0,	0,	0,	0,	0x0982, /* ANUSVARA */
  GDK_y,	0,	0,	0,	0,	0x09AC, /* LETTER BA */
/* GDK_z,	0,	0,	0,	0,	0x0993,  */

  GDK_braceleft,	0,	0,	0,	0,	0x9A2, /* LETTER DDHA     */
/* GDK_bar,		0,	0,	0,	0,	0x09,    */
  GDK_braceright,	0,	0,	0,	0,	0x99E, /* LETTER NYA      */
/* GDK_asciitilde,	0,	0,	0,	0,	0x09		*/  
};

static void
inscript_bng_im_class_init (GtkIMContextSimpleClass *class)
{
}

static void
inscript_bng_im_init (GtkIMContextSimple *im_context)
{
  gtk_im_context_simple_add_table (im_context,
				   inscript_bng_compose_seqs,
				   4,
				   G_N_ELEMENTS (inscript_bng_compose_seqs) / (4 + 2));
}

static const GtkIMContextInfo inscript_bng_im_info = { 
  INSCRIPT_BNG_IM_ID,		           /* ID */
  N_("Bengali (Inscript)"),        /* Human readable name */
  "gtk+",			   /* Translation domain */
   GTK_LOCALEDIR,		   /* Dir for bindtextdomain (not strictly needed for "gtk+") */
  ""			           /* Languages for which this module is the default */
};

static const GtkIMContextInfo *info_list[] = {
  &inscript_bng_im_info
};

void
im_module_init (GTypeModule *module)
{
  inscript_bng_im_register_type (module);
}

void 
im_module_exit (void)
{
}

void 
im_module_list (const GtkIMContextInfo ***contexts,
		int                      *n_contexts)
{
  *contexts = info_list;
  *n_contexts = G_N_ELEMENTS (info_list);
}

GtkIMContext *
im_module_create (const gchar *context_id)
{
  if (strcmp (context_id, INSCRIPT_BNG_IM_ID) == 0)
    return GTK_IM_CONTEXT (gtk_type_new (type_inscript_bng_im));
  else
    return NULL;
}
