/* GTK - The GIMP Toolkit
 * Copyright (C) 2000 Red Hat Software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 * Inscript Keymap for Gujarati
 * 
 * Author(s)  G Karunakar  <karunakarg@hotmail.com>
 *  
 */

#include <string.h>

#include <gdk/gdkkeysyms.h>

#include "gtk/gtkintl.h"
#include "gtk/gtkimcontextsimple.h"
#include "gtk/gtkimmodule.h"

#define INSCRIPT_GUJ_IM_ID  "inscript_guj_im"

GType type_inscript_guj_im = 0;

static void inscript_guj_im_class_init (GtkIMContextSimpleClass *class);
static void inscript_guj_im_init (GtkIMContextSimple *im_context);

static void
inscript_guj_im_register_type (GTypeModule *module)
{
  static const GTypeInfo object_info =
  {
    sizeof (GtkIMContextSimpleClass),
    (GBaseInitFunc) NULL,
    (GBaseFinalizeFunc) NULL,
    (GClassInitFunc) inscript_guj_im_class_init,
    NULL,           /* class_finalize */
    NULL,           /* class_data */
    sizeof (GtkIMContextSimple),
    0,
    (GtkObjectInitFunc) inscript_guj_im_init,
  };

  type_inscript_guj_im = 
    g_type_module_register_type (module,
				 GTK_TYPE_IM_CONTEXT_SIMPLE,
				 "GtkIMContextGujarati",
				 &object_info, 0);
}


/* Based on the Inscript keyboard layout for Gujarati
 *  Commented Keysyms are unassigned, leaving them so that can assign later on
 *  Some conjuncts are also assigned to keys, not yet figured how to get them
 */
 
static guint16 inscript_guj_compose_seqs[] = {

  GDK_exclam,		0,	0,	0,	0,	0xA8D, /* LETTER CANDRA E */
  GDK_quotedbl,		0,	0,	0,	0,	0xAA0, /* LETTER TTHA */
/*  GDK_numbersign,	0,	0,	0,	0,	0x9,  VIRAMA + LETTER RA */
/*  GDK_dollar,		0,	0,	0,	0,	0x9,  LETTER RA + VIRAMA */
/*  GDK_percent,	0,	0,	0,	0,	0x9,  LETTER JA + VIRAMA + LETTER NYA */
/*  GDK_ampersand,	0,	0,	0,	0,	0x9,  LETTER KA + VIRAMA + LETTER SSA */
/*  ?GDK_apostrophe,	0,	0,	0,	0,	0x9,   */
/*  ?GDK_parenleft,	0,	0,	0,	0,	0x9,   */
/*  ?GDK_parenright,	0,	0,	0,	0,	0x9,   */
/*  GDK_asterisk,	0,	0,	0,	0,	0x9,  LETTER SHA + VIRAMA + LETTER RA */
  GDK_plus,		0,	0,	0,	0,	0xA8B, /* LETTER VOC R */
/*  ?GDK_comma,		0,	0,	0,	0,	0x9,   */
/*  ?GDK_minus,		0,	0,	0,	0,	0x9,   */
/*  ?GDK_period,	0,	0,	0,	0,	0x9,   */
  GDK_slash,		0,	0,	0,	0,	0xAAF, /* LETTER YA */

  GDK_0,	0,	0,	0,	0,	0x0AE6,  /* DIGIT 0 */
  GDK_1,	0,	0,	0,	0,	0x0AE7,  /* DIGIT 1 */ 
  GDK_2,	0,	0,	0,	0,	0x0AE8,  /* DIGIT 2 */ 
  GDK_3,	0,	0,	0,	0,	0x0AE9,  /* DIGIT 3 */ 
  GDK_4,	0,	0,	0,	0,	0x0AEA,  /* DIGIT 4 */ 
  GDK_5,	0,	0,	0,	0,	0x0AEB,  /* DIGIT 5 */ 
  GDK_6,	0,	0,	0,	0,	0x0AEC,  /* DIGIT 6 */ 
  GDK_7,	0,	0,	0,	0,	0x0AED,  /* DIGIT 7 */ 
  GDK_8,	0,	0,	0,	0,	0x0AEE,  /* DIGIT 8 */ 
  GDK_9,	0,	0,	0,	0,	0x0AEF,  /* DIGIT 9 */ 
  
  GDK_colon,		0,	0,	0,	0,	0xA9B, /* LETTER CHA */
  GDK_semicolon,	0,	0,	0,	0,	0xA9A, /* LETTER CA */
  GDK_less,		0,	0,	0,	0,	0xAB7, /* LETTER SSA */
  GDK_equal,		0,	0,	0,	0,	0xAC3, /* VS VOC R */
/* ?GDK_greater,	0,	0,	0,	0,	0x964,  DEV DANDA */
/* ?GDK_question,	0,	0,	0,	0,	0x9,	*/
/* ?GDK_at,		0,	0,	0,	0,	0x9,	*/

  
  GDK_A,	0,	0,	0,	0,	0x0A93, /* LETTER O */
/* ?GDK_B,	0,	0,	0,	0,	0x0935,  */
  GDK_C,	0,	0,	0,	0,	0x0AA3, /* LETTER NNA */
  GDK_D,	0,	0,	0,	0,	0x0A85, /* LETTER A */
  GDK_E,	0,	0,	0,	0,	0x0A86, /* LETTER AA */
  GDK_F,	0,	0,	0,	0,	0x0A87, /* LETTER I */
  GDK_G,	0,	0,	0,	0,	0x0A89, /* LETTER U */
  GDK_H,	0,	0,	0,	0,	0x0AAB, /* LETTER PHA */
  GDK_I,	0,	0,	0,	0,	0x0A98, /* LETTER GHA */
/* ?GDK_J,	0,	0,	0,	0,	0x0931,  LETTER RRA */
  GDK_K,	0,	0,	0,	0,	0x0A96, /* LETTER KHA */
  GDK_L,	0,	0,	0,	0,	0x0AA5, /* LETTER THA */
  GDK_M,	0,	0,	0,	0,	0x0AB6, /* LETTER SHA */
  GDK_N,	0,	0,	0,	0,	0x0AB3, /* LETTER LLA */
  GDK_O,	0,	0,	0,	0,	0x0AA7, /* LETTER DHA */
  GDK_P,	0,	0,	0,	0,	0x0A9D, /* LETTER JHA */
  GDK_Q,	0,	0,	0,	0,	0x0A94, /* LETTER AU */
  GDK_R,	0,	0,	0,	0,	0x0A88, /* LETTER II */
  GDK_S,	0,	0,	0,	0,	0x0A8F, /* LETTER E */
  GDK_T,	0,	0,	0,	0,	0x0A8A, /* LETTER UU */
  GDK_U,	0,	0,	0,	0,	0x0A99, /* LETTER NGA */
/* ?GDK_V,	0,	0,	0,	0,	0x0913,  */
  GDK_W,	0,	0,	0,	0,	0x0A90, /* LETTER AI */
  GDK_X,	0,	0,	0,	0,	0x0A81, /* CANDRABINDU */
  GDK_Y,	0,	0,	0,	0,	0x0AAD, /* LETTER BHA */
/* ?GDK_Z,	0,	0,	0,	0,	0x0913,  */

  GDK_bracketleft,	0,	0,	0,	0,	0xAA1, /* LETTER DDA */
  GDK_backslash,	0,	0,	0,	0,	0xAC9, /* VS CANDRA O */
  GDK_bracketright,	0,	0,	0,	0,	0xABC, /* VS NUKTA */
/* ?GDK_asciicircum,	0,	0,	0,	0,	0xA,	*/
  GDK_underscore,	0,	0,	0,	0,	0xA83, /* LETTER VISARGA */
/* ?GDK_grave,		0,	0,	0,	0,	0xA,	*/  

  GDK_a,	0,	0,	0,	0,	0x0ACB, /* VS O */
  GDK_b,	0,	0,	0,	0,	0x0AB5, /* LETTER VA */
  GDK_c,	0,	0,	0,	0,	0x0AAE, /* LETTER MA */
  GDK_d,	0,	0,	0,	0,	0x0ACD, /* VIRAMA */
  GDK_e,	0,	0,	0,	0,	0x0ABE, /* VS AA */
  GDK_f,	0,	0,	0,	0,	0x0ABF, /* VS I */
  GDK_g,	0,	0,	0,	0,	0x0AC1, /* VS U */
  GDK_h,	0,	0,	0,	0,	0x0AAA, /* LETTER PA */
  GDK_i,	0,	0,	0,	0,	0x0A97, /* LETTER GA */
  GDK_j,	0,	0,	0,	0,	0x0AB0, /* LETTER RA */
  GDK_k,	0,	0,	0,	0,	0x0A95, /* LETTER KA */
  GDK_l,	0,	0,	0,	0,	0x0AA4, /* LETTER TA */
  GDK_m,	0,	0,	0,	0,	0x0AB8, /* LETTER SA */
  GDK_n,	0,	0,	0,	0,	0x0AB2, /* LETTER LA */
  GDK_o,	0,	0,	0,	0,	0x0AA6, /* LETTER DA */
  GDK_p,	0,	0,	0,	0,	0x0A9C, /* LETTER JA */
  GDK_q,	0,	0,	0,	0,	0x0A94, /* LETTER AU */
  GDK_r,	0,	0,	0,	0,	0x0AC0, /* VS II */
  GDK_s,	0,	0,	0,	0,	0x0AC7, /* VS E */
  GDK_t,	0,	0,	0,	0,	0x0AC2, /* VS UU */
  GDK_u,	0,	0,	0,	0,	0x0AB9, /* LETTER HA */
  GDK_v,	0,	0,	0,	0,	0x0AA8, /* LETTER NA */
  GDK_w,	0,	0,	0,	0,	0x0AC8, /* VS AI */
  GDK_x,	0,	0,	0,	0,	0x0A82, /* ANUSVARA */
  GDK_y,	0,	0,	0,	0,	0x0AAC, /* LETTER BA */
/* ?GDK_z,	0,	0,	0,	0,	0x0913,  */

  GDK_braceleft,	0,	0,	0,	0,	0xAA2, /* LETTER DDHA */
  GDK_bar,		0,	0,	0,	0,	0xA91, /* LETTER CANDRA O */
  GDK_braceright,	0,	0,	0,	0,	0xA9E, /* LETTER NYA */
/* ?GDK_asciitilde,	0,	0,	0,	0,	0xA	 	*/ 
};

static void
inscript_guj_im_class_init (GtkIMContextSimpleClass *class)
{
}

static void
inscript_guj_im_init (GtkIMContextSimple *im_context)
{
  gtk_im_context_simple_add_table (im_context,
				   inscript_guj_compose_seqs,
				   4,
				   G_N_ELEMENTS (inscript_guj_compose_seqs) / (4 + 2));
}

static const GtkIMContextInfo inscript_guj_im_info = { 
  INSCRIPT_GUJ_IM_ID,		           /* ID */
  N_("Gujarati (Inscript)"),        /* Human readable name */
  "gtk+",			   /* Translation domain */
   GTK_LOCALEDIR,		   /* Dir for bindtextdomain (not strictly needed for "gtk+") */
  ""			           /* Languages for which this module is the default */
};

static const GtkIMContextInfo *info_list[] = {
  &inscript_guj_im_info
};

void
im_module_init (GTypeModule *module)
{
  inscript_guj_im_register_type (module);
}

void 
im_module_exit (void)
{
}

void 
im_module_list (const GtkIMContextInfo ***contexts,
		int                      *n_contexts)
{
  *contexts = info_list;
  *n_contexts = G_N_ELEMENTS (info_list);
}

GtkIMContext *
im_module_create (const gchar *context_id)
{
  if (strcmp (context_id, INSCRIPT_GUJ_IM_ID) == 0)
    return GTK_IM_CONTEXT (gtk_type_new (type_inscript_guj_im));
  else
    return NULL;
}

