/* GTK - The GIMP Toolkit
 * Copyright (C) 2000 Red Hat Software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 * Inscript Keymap for Gurmukhi (Punjabi)
 * 
 * Author(s)  G Karunakar  <karunakarg@hotmail.com>
 *  
 */

#include <string.h>

#include <gdk/gdkkeysyms.h>

#include "gtk/gtkintl.h"
#include "gtk/gtkimcontextsimple.h"
#include "gtk/gtkimmodule.h"

#define INSCRIPT_GUR_IM_ID  "inscript_gur_im"

GType type_inscript_gur_im = 0;

static void inscript_gur_im_class_init (GtkIMContextSimpleClass *class);
static void inscript_gur_im_init (GtkIMContextSimple *im_context);

static void
inscript_gur_im_register_type (GTypeModule *module)
{
  static const GTypeInfo object_info =
  {
    sizeof (GtkIMContextSimpleClass),
    (GBaseInitFunc) NULL,
    (GBaseFinalizeFunc) NULL,
    (GClassInitFunc) inscript_gur_im_class_init,
    NULL,           /* class_finalize */
    NULL,           /* class_data */
    sizeof (GtkIMContextSimple),
    0,
    (GtkObjectInitFunc) inscript_gur_im_init,
  };

  type_inscript_gur_im = 
    g_type_module_register_type (module,
				 GTK_TYPE_IM_CONTEXT_SIMPLE,
				 "GtkIMContextGurmukhi",
				 &object_info, 0);
}


/* Based on the Inscript keyboard layout for gurmukhi
 *  Commented Keysyms are unassigned, leaving them so that can assign later on
 *  Some conjuncts are also assigned to keys, not yet figured how to get them
 */

static guint16 inscript_gur_compose_seqs[] = {

/* GDK_exclam,		0,	0,	0,	0,	0x0A,    */
  GDK_quotedbl,		0,	0,	0,	0,	0x0A20, /* LETTER TTHA */
  GDK_numbersign,	0,	0,	0,	0,	0x0A4D, /* VIRAMA */
/* GDK_dollar,		0,	0,	0,	0,	0x0A,  LETTER RA + VIRAMA */
/* GDK_percent,		0,	0,	0,	0,	0x0A,  LETTER JA + VIRAMA + LETTER NYA */
/* GDK_ampersand,	0,	0,	0,	0,	0x0A,  LETTER KA + VIRAMA + LETTER SSA */
  GDK_apostrophe,	0,	0,	0,	0,	0x0A1F, /* LETTER TTA  */
/* GDK_parenleft,	0,	0,	0,	0,	0x0A,   */
/* GDK_parenright,	0,	0,	0,	0,	0x0A,   */
/* GDK_asterisk,	0,	0,	0,	0,	0x0A,  LETTER SHA + VIRAMA + LETTER RA */
/* GDK_plus,		0,	0,	0,	0,	0x0A,   */
/* GDK_comma,		0,	0,	0,	0,	0x0A,   */
/* GDK_minus,		0,	0,	0,	0,	0x0A,   */
/* GDK_period,		0,	0,	0,	0,	0x0A,   */
   GDK_slash,		0,	0,	0,	0,	0x0A2F, /* LETTER YA */

  GDK_0,	0,	0,	0,	0,	0x0A66, /* DIGIT 0 */
  GDK_1,	0,	0,	0,	0,	0x0A67, /* DIGIT 1 */ 
  GDK_2,	0,	0,	0,	0,	0x0A68, /* DIGIT 2 */ 
  GDK_3,	0,	0,	0,	0,	0x0A69, /* DIGIT 3 */ 
  GDK_4,	0,	0,	0,	0,	0x0A6A, /* DIGIT 4 */ 
  GDK_5,	0,	0,	0,	0,	0x0A6B, /* DIGIT 5 */ 
  GDK_6,	0,	0,	0,	0,	0x0A6C, /* DIGIT 6 */ 
  GDK_7,	0,	0,	0,	0,	0x0A6D, /* DIGIT 7 */ 
  GDK_8,	0,	0,	0,	0,	0x0A6E, /* DIGIT 8 */ 
  GDK_9,	0,	0,	0,	0,	0x0A6F, /* DIGIT 9 */ 
  
  GDK_colon,		0,	0,	0,	0,	0x0A1B, /* LETTER CHA */
  GDK_semicolon,	0,	0,	0,	0,	0x0A1A, /* LETTER CA */
/* GDK_less,		0,	0,	0,	0,	0x0A,    */
/* GDK_equal,		0,	0,	0,	0,	0x0A,    */
  GDK_greater,		0,	0,	0,	0,	0x0A,   /* DEV DANDA */
/* GDK_question,	0,	0,	0,	0,	0x0A,    */
/* GDK_at,		0,	0,	0,	0,	0x0A,    */
  
  GDK_A,	0,	0,	0,	0,	0x09A3, /* LETTER OO */
/* GDK_B,	0,	0,	0,	0,	0x0A,    */
  GDK_C,	0,	0,	0,	0,	0x0A23, /* LETTER NNA */
  GDK_D,	0,	0,	0,	0,	0x0A05, /* LETTER A */
  GDK_E,	0,	0,	0,	0,	0x0A06, /* LETTER AA */
  GDK_F,	0,	0,	0,	0,	0x0A07, /* LETTER I */
  GDK_G,	0,	0,	0,	0,	0x0A09, /* LETTER U */
  GDK_H,	0,	0,	0,	0,	0x0A2B, /* LETTER PHA */
  GDK_I,	0,	0,	0,	0,	0x0A18, /* LETTER GHA */
/* GDK_J,	0,	0,	0,	0,	0x0A,    */
  GDK_K,	0,	0,	0,	0,	0x0A16, /* LETTER KHA */
  GDK_L,	0,	0,	0,	0,	0x0A25, /* LETTER THA */
  GDK_M,	0,	0,	0,	0,	0x0A36, /* LETTER SHA */
  GDK_N,	0,	0,	0,	0,	0x0A33, /* LETTER LLA */
  GDK_O,	0,	0,	0,	0,	0x0A27, /* LETTER DHA */
  GDK_P,	0,	0,	0,	0,	0x0A1D, /* LETTER JHA */
  GDK_Q,	0,	0,	0,	0,	0x0A14, /* LETTER AU */
  GDK_R,	0,	0,	0,	0,	0x0A08, /* LETTER II */
  GDK_S,	0,	0,	0,	0,	0x0A0F, /* LETTER EE */
  GDK_T,	0,	0,	0,	0,	0x0A0A, /* LETTER UU */
  GDK_U,	0,	0,	0,	0,	0x0A19, /* LETTER NGA */
/* GDK_V,	0,	0,	0,	0,	0x0A,    */
  GDK_W,	0,	0,	0,	0,	0x0A10, /* LETTER AI */
/* GDK_X,	0,	0,	0,	0,	0x0A,    */
  GDK_Y,	0,	0,	0,	0,	0x0A2D, /* LETTER BHA */
/* GDK_Z,	0,	0,	0,	0,	0x0A,    */

  GDK_bracketleft,	0,	0,	0,	0,	0x0A21, /* LETTER DDA */
/* GDK_backslash,	0,	0,	0,	0,	0x0A,    */
  GDK_bracketright,	0,	0,	0,	0,	0x0A3C, /* VS NUKTA */
/* GDK_asciicircum,	0,	0,	0,	0,	0x0A,    */
/* GDK_underscore,	0,	0,	0,	0,	0x0A,    */
/* GDK_grave,		0,	0,	0,	0,	0x0A,    */
  
  GDK_a,	0,	0,	0,	0,	0x0A4B, /* VS OO */
  GDK_b,	0,	0,	0,	0,	0x0A35, /* LETTER VA */
  GDK_c,	0,	0,	0,	0,	0x0A2E, /* LETTER MA */
  GDK_d,	0,	0,	0,	0,	0x0A4D, /* VIRAMA */
  GDK_e,	0,	0,	0,	0,	0x0A3E, /* VS AA */
  GDK_f,	0,	0,	0,	0,	0x0A3F, /* VS I */
  GDK_g,	0,	0,	0,	0,	0x0A41, /* VS U */
  GDK_h,	0,	0,	0,	0,	0x0A2A, /* LETTER PA */
  GDK_i,	0,	0,	0,	0,	0x0A17, /* LETTER GA */
  GDK_j,	0,	0,	0,	0,	0x0A30, /* LETTER RA */
  GDK_k,	0,	0,	0,	0,	0x0A15, /* LETTER KA */
  GDK_l,	0,	0,	0,	0,	0x0A24, /* LETTER TA */
  GDK_m,	0,	0,	0,	0,	0x0A38, /* LETTER SA */
  GDK_n,	0,	0,	0,	0,	0x0A32, /* LETTER LA */
  GDK_o,	0,	0,	0,	0,	0x0A26, /* LETTER DA */
  GDK_p,	0,	0,	0,	0,	0x0A1C, /* LETTER JA */
  GDK_q,	0,	0,	0,	0,	0x0A14, /* LETTER AU */
  GDK_r,	0,	0,	0,	0,	0x0A40, /* VS II */
  GDK_s,	0,	0,	0,	0,	0x0A46, /* VS E */
  GDK_t,	0,	0,	0,	0,	0x0A42, /* VS UU */
  GDK_u,	0,	0,	0,	0,	0x0A39, /* LETTER HA */
  GDK_v,	0,	0,	0,	0,	0x0A28, /* LETTER NA */
  GDK_w,	0,	0,	0,	0,	0x0A48, /* VS AI */
  GDK_x,	0,	0,	0,	0,	0x0A70, /* TIPPI */
  GDK_y,	0,	0,	0,	0,	0x0A2C, /* LETTER BA */
/* GDK_z,	0,	0,	0,	0,	0x0A,    */

  GDK_braceleft,	0,	0,	0,	0,	0x0A22, /* LETTER DDHA */
/* GDK_bar,		0,	0,	0,	0,	0x0A     */
  GDK_braceright,	0,	0,	0,	0,	0x0A1E, /* LETTER NYA */
/* GDK_asciitilde,	0,	0,	0,	0,	0x0A,    */

};

static void
inscript_gur_im_class_init (GtkIMContextSimpleClass *class)
{
}

static void
inscript_gur_im_init (GtkIMContextSimple *im_context)
{
  gtk_im_context_simple_add_table (im_context,
				   inscript_gur_compose_seqs,
				   4,
				   G_N_ELEMENTS (inscript_gur_compose_seqs) / (4 + 2));
}

static const GtkIMContextInfo inscript_gur_im_info = { 
  INSCRIPT_GUR_IM_ID,		           /* ID */
  N_("Gurmukhi (Inscript)"),        /* Human readable name */
  "gtk+",			   /* Translation domain */
   GTK_LOCALEDIR,		   /* Dir for bindtextdomain (not strictly needed for "gtk+") */
  ""			           /* Languages for which this module is the default */
};

static const GtkIMContextInfo *info_list[] = {
  &inscript_gur_im_info
};

void
im_module_init (GTypeModule *module)
{
  inscript_gur_im_register_type (module);
}

void 
im_module_exit (void)
{
}

void 
im_module_list (const GtkIMContextInfo ***contexts,
		int                      *n_contexts)
{
  *contexts = info_list;
  *n_contexts = G_N_ELEMENTS (info_list);
}

GtkIMContext *
im_module_create (const gchar *context_id)
{
  if (strcmp (context_id, INSCRIPT_GUR_IM_ID) == 0)
    return GTK_IM_CONTEXT (gtk_type_new (type_inscript_gur_im));
  else
    return NULL;
}
